@@ -2,6 +2,9 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
+//go:build go1.15
+// +build go1.15
+
 package hooks
 
 import (
@@ -34,10 +37,6 @@ func updateAnalyzers(options *source.Options) {
 	// Always add hooks for all available analyzers, but disable them if the
 	// user does not have staticcheck enabled (they may enable it later on).
 	for _, a := range analyzers {
-		addStaticcheckAnalyzer(options, a)
+		options.AddStaticcheckAnalyzer(a)
 	}
 }
-
-func addStaticcheckAnalyzer(options *source.Options, a *analysis.Analyzer) {
-	options.StaticcheckAnalyzers[a.Name] = source.Analyzer{Analyzer: a, Enabled: true}
-}
