@@ -11,16 +11,22 @@ import (
 	"go/ast"
 	"go/types"
 	"path"
+	"path/filepath"
+	"regexp"
 	"sort"
 	"strings"
 	"sync"
 
 	"golang.org/x/mod/module"
+	"golang.org/x/tools/go/ast/astutil"
 	"golang.org/x/tools/go/packages"
 	"golang.org/x/tools/internal/event"
+	"golang.org/x/tools/internal/lsp/command"
 	"golang.org/x/tools/internal/lsp/debug/tag"
+	"golang.org/x/tools/internal/lsp/protocol"
 	"golang.org/x/tools/internal/lsp/source"
 	"golang.org/x/tools/internal/memoize"
+	"golang.org/x/tools/internal/packagesinternal"
 	"golang.org/x/tools/internal/span"
 	"golang.org/x/tools/internal/typesinternal"
 	errors "golang.org/x/xerrors"
@@ -99,7 +105,7 @@ func (s *snapshot) buildPackageHandle(ctx context.Context, id packageID, mode so
 		wg.Wait()
 
 		return data
-	})
+	}, nil)
 	ph.handle = h
 
 	// Cache the handle in the snapshot. If a package handle has already
@@ -144,7 +150,7 @@ func (s *snapshot) buildKey(ctx context.Context, id packageID, mode source.Parse
 	for _, depID := range depList {
 		depHandle, err := s.buildPackageHandle(ctx, depID, s.workspaceParseMode(depID))
 		if err != nil {
-			event.Error(ctx, "no dep handle", err, tag.Package.Of(string(depID)))
+			event.Error(ctx, fmt.Sprintf("%s: no dep handle for %s", id, depID), err, tag.Snapshot.Of(s.id))
 			if ctx.Err() != nil {
 				return nil, nil, ctx.Err()
 			}
@@ -156,7 +162,8 @@ func (s *snapshot) buildKey(ctx context.Context, id packageID, mode source.Parse
 		deps[depHandle.m.pkgPath] = depHandle
 		depKeys = append(depKeys, depHandle.key)
 	}
-	ph.key = checkPackageKey(ctx, ph.m.id, compiledGoFiles, m.config, depKeys, mode)
+	experimentalKey := s.View().Options().ExperimentalPackageCacheKey
+	ph.key = checkPackageKey(ctx, ph.m.id, compiledGoFiles, m.config, depKeys, mode, experimentalKey)
 	return ph, deps, nil
 }
 
@@ -168,10 +175,16 @@ func (s *snapshot) workspaceParseMode(id packageID) source.ParseMode {
 	}
 }
 
-func checkPackageKey(ctx context.Context, id packageID, pghs []*parseGoHandle, cfg *packages.Config, deps []packageHandleKey, mode source.ParseMode) packageHandleKey {
+func checkPackageKey(ctx context.Context, id packageID, pghs []*parseGoHandle, cfg *packages.Config, deps []packageHandleKey, mode source.ParseMode, experimentalKey bool) packageHandleKey {
 	b := bytes.NewBuffer(nil)
 	b.WriteString(string(id))
-	b.WriteString(hashConfig(cfg))
+	if !experimentalKey {
+		// cfg was used to produce the other hashed inputs (package ID, parsed Go
+		// files, and deps). It should not otherwise affect the inputs to the type
+		// checker, so this experiment omits it. This should increase cache hits on
+		// the daemon as cfg contains the environment and working directory.
+		b.WriteString(hashConfig(cfg))
+	}
 	b.WriteByte(byte(mode))
 	for _, dep := range deps {
 		b.WriteString(string(dep))
@@ -182,6 +195,19 @@ func checkPackageKey(ctx context.Context, id packageID, pghs []*parseGoHandle, c
 	return packageHandleKey(hashContents(b.Bytes()))
 }
 
+// hashEnv returns a hash of the snapshot's configuration.
+func hashEnv(s *snapshot) string {
+	s.view.optionsMu.Lock()
+	env := s.view.options.EnvSlice()
+	s.view.optionsMu.Unlock()
+
+	b := &bytes.Buffer{}
+	for _, e := range env {
+		b.WriteString(e)
+	}
+	return hashContents(b.Bytes())
+}
+
 // hashConfig returns the hash for the *packages.Config.
 func hashConfig(config *packages.Config) string {
 	b := bytes.NewBuffer(nil)
@@ -271,7 +297,7 @@ func typeCheck(ctx context.Context, snapshot *snapshot, m *metadata, mode source
 	// meaningless, and we don't want clients to access it.
 	if m.module != nil {
 		version := m.module.Version
-		if version == source.WorkspaceModuleVersion {
+		if source.IsWorkspaceModuleVersion(version) {
 			version = ""
 		}
 		pkg.version = &module.Version{
@@ -356,17 +382,17 @@ func typeCheck(ctx context.Context, snapshot *snapshot, m *metadata, mode source
 		// Try to attach errors messages to the file as much as possible.
 		var found bool
 		for _, e := range rawErrors {
-			srcErr, err := sourceError(ctx, snapshot, pkg, e)
+			srcDiags, err := sourceDiagnostics(ctx, snapshot, pkg, protocol.SeverityError, e)
 			if err != nil {
 				continue
 			}
 			found = true
-			pkg.errors = append(pkg.errors, srcErr)
+			pkg.diagnostics = append(pkg.diagnostics, srcDiags...)
 		}
 		if found {
 			return pkg, nil
 		}
-		return nil, errors.Errorf("no parsed files for package %s, expected: %s, list errors: %v", pkg.m.pkgPath, pkg.compiledGoFiles, rawErrors)
+		return nil, errors.Errorf("no parsed files for package %s, expected: %v, list errors: %v", pkg.m.pkgPath, pkg.compiledGoFiles, rawErrors)
 	} else {
 		pkg.types = types.NewPackage(string(m.pkgPath), string(m.name))
 	}
@@ -387,7 +413,7 @@ func typeCheck(ctx context.Context, snapshot *snapshot, m *metadata, mode source
 			}
 			dep := resolveImportPath(pkgPath, pkg, deps)
 			if dep == nil {
-				return nil, errors.Errorf("no package for import %s", pkgPath)
+				return nil, snapshot.missingPkgError(pkgPath)
 			}
 			if !isValidImport(m.pkgPath, dep.m.pkgPath) {
 				return nil, errors.Errorf("invalid use of internal package %s", pkgPath)
@@ -418,21 +444,197 @@ func typeCheck(ctx context.Context, snapshot *snapshot, m *metadata, mode source
 	if mode == source.ParseFull {
 		expandErrors(rawErrors)
 		for _, e := range rawErrors {
-			srcErr, err := sourceError(ctx, snapshot, pkg, e)
+			srcDiags, err := sourceDiagnostics(ctx, snapshot, pkg, protocol.SeverityError, e)
 			if err != nil {
 				event.Error(ctx, "unable to compute error positions", err, tag.Package.Of(pkg.ID()))
 				continue
 			}
-			pkg.errors = append(pkg.errors, srcErr)
+			pkg.diagnostics = append(pkg.diagnostics, srcDiags...)
+
 			if err, ok := e.(extendedError); ok {
 				pkg.typeErrors = append(pkg.typeErrors, err.primary)
 			}
 		}
+
+		depsErrors, err := snapshot.depsErrors(ctx, pkg)
+		if err != nil {
+			return nil, err
+		}
+		pkg.diagnostics = append(pkg.diagnostics, depsErrors...)
+		if err := addGoGetFixes(ctx, snapshot, pkg); err != nil {
+			return nil, err
+		}
 	}
 
 	return pkg, nil
 }
 
+var importErrorRe = regexp.MustCompile(`could not import ([^\s]+)`)
+var missingModuleErrorRe = regexp.MustCompile(`cannot find module providing package ([^\s:]+)`)
+
+func addGoGetFixes(ctx context.Context, snapshot source.Snapshot, pkg *pkg) error {
+	if len(pkg.compiledGoFiles) == 0 || snapshot.GoModForFile(pkg.compiledGoFiles[0].URI) == "" {
+		// Go get only supports module mode for now.
+		return nil
+	}
+	for _, diag := range pkg.diagnostics {
+		matches := importErrorRe.FindStringSubmatch(diag.Message)
+		if len(matches) == 0 {
+			matches = missingModuleErrorRe.FindStringSubmatch(diag.Message)
+		}
+		if len(matches) == 0 {
+			continue
+		}
+		direct := !strings.Contains(diag.Message, "error while importing")
+		title := fmt.Sprintf("go get package %v", matches[1])
+		cmd, err := command.NewGoGetPackageCommand(title, command.GoGetPackageArgs{
+			URI:        protocol.URIFromSpanURI(pkg.compiledGoFiles[0].URI),
+			AddRequire: direct,
+			Pkg:        matches[1],
+		})
+		if err != nil {
+			return err
+		}
+		diag.SuggestedFixes = append(diag.SuggestedFixes, source.SuggestedFixFromCommand(cmd))
+	}
+	return nil
+}
+
+func (s *snapshot) depsErrors(ctx context.Context, pkg *pkg) ([]*source.Diagnostic, error) {
+	// Select packages that can't be found, and were imported in non-workspace packages.
+	// Workspace packages already show their own errors.
+	var relevantErrors []*packagesinternal.PackageError
+	for _, depsError := range pkg.m.depsErrors {
+		// Up to Go 1.15, the missing package was included in the stack, which
+		// was presumably a bug. We want the next one up.
+		directImporterIdx := len(depsError.ImportStack) - 1
+		if s.view.goversion < 15 {
+			directImporterIdx = len(depsError.ImportStack) - 2
+		}
+		if directImporterIdx < 0 {
+			continue
+		}
+
+		directImporter := depsError.ImportStack[directImporterIdx]
+		if _, ok := s.isWorkspacePackage(packageID(directImporter)); ok {
+			continue
+		}
+		relevantErrors = append(relevantErrors, depsError)
+	}
+
+	// Don't build the import index for nothing.
+	if len(relevantErrors) == 0 {
+		return nil, nil
+	}
+
+	// Build an index of all imports in the package.
+	type fileImport struct {
+		cgf *source.ParsedGoFile
+		imp *ast.ImportSpec
+	}
+	allImports := map[string][]fileImport{}
+	for _, cgf := range pkg.compiledGoFiles {
+		for _, group := range astutil.Imports(s.FileSet(), cgf.File) {
+			for _, imp := range group {
+				if imp.Path == nil {
+					continue
+				}
+				path := strings.Trim(imp.Path.Value, `"`)
+				allImports[path] = append(allImports[path], fileImport{cgf, imp})
+			}
+		}
+	}
+
+	// Apply a diagnostic to any import involved in the error, stopping after
+	// we reach the workspace.
+	var errors []*source.Diagnostic
+	for _, depErr := range relevantErrors {
+		for i := len(depErr.ImportStack) - 1; i >= 0; i-- {
+			item := depErr.ImportStack[i]
+			for _, imp := range allImports[item] {
+				rng, err := source.NewMappedRange(s.FileSet(), imp.cgf.Mapper, imp.imp.Pos(), imp.imp.End()).Range()
+				if err != nil {
+					return nil, err
+				}
+				errors = append(errors, &source.Diagnostic{
+					URI:      imp.cgf.URI,
+					Range:    rng,
+					Severity: protocol.SeverityError,
+					Source:   source.TypeError,
+					Message:  fmt.Sprintf("error while importing %v: %v", item, depErr.Err),
+				})
+			}
+			if _, ok := s.isWorkspacePackage(packageID(item)); ok {
+				break
+			}
+		}
+	}
+
+	if len(pkg.compiledGoFiles) == 0 {
+		return errors, nil
+	}
+	mod := s.GoModForFile(pkg.compiledGoFiles[0].URI)
+	if mod == "" {
+		return errors, nil
+	}
+	fh, err := s.GetFile(ctx, mod)
+	if err != nil {
+		return nil, err
+	}
+	pm, err := s.ParseMod(ctx, fh)
+	if err != nil {
+		return nil, err
+	}
+
+	// Add a diagnostic to the module that contained the lowest-level import of
+	// the missing package.
+	for _, depErr := range relevantErrors {
+		for i := len(depErr.ImportStack) - 1; i >= 0; i-- {
+			item := depErr.ImportStack[i]
+			m := s.getMetadata(packageID(item))
+			if m == nil || m.module == nil {
+				continue
+			}
+			modVer := module.Version{Path: m.module.Path, Version: m.module.Version}
+			reference := findModuleReference(pm.File, modVer)
+			if reference == nil {
+				continue
+			}
+			rng, err := rangeFromPositions(pm.Mapper, reference.Start, reference.End)
+			if err != nil {
+				return nil, err
+			}
+			errors = append(errors, &source.Diagnostic{
+				URI:      pm.URI,
+				Range:    rng,
+				Severity: protocol.SeverityError,
+				Source:   source.TypeError,
+				Message:  fmt.Sprintf("error while importing %v: %v", item, depErr.Err),
+			})
+			break
+		}
+	}
+	return errors, nil
+}
+
+// missingPkgError returns an error message for a missing package that varies
+// based on the user's workspace mode.
+func (s *snapshot) missingPkgError(pkgPath string) error {
+	if s.workspaceMode()&moduleMode != 0 {
+		return fmt.Errorf("no required module provides package %q", pkgPath)
+	}
+	gorootSrcPkg := filepath.FromSlash(filepath.Join(s.view.goroot, "src", pkgPath))
+
+	var b strings.Builder
+	b.WriteString(fmt.Sprintf("cannot find package %q in any of \n\t%s (from $GOROOT)", pkgPath, gorootSrcPkg))
+
+	for _, gopath := range strings.Split(s.view.gopath, ":") {
+		gopathSrcPkg := filepath.FromSlash(filepath.Join(gopath, "src", pkgPath))
+		b.WriteString(fmt.Sprintf("\n\t%s (from $GOPATH)", gopathSrcPkg))
+	}
+	return errors.New(b.String())
+}
+
 type extendedError struct {
 	primary     types.Error
 	secondaries []types.Error
