@@ -41,6 +41,7 @@ type metadata struct {
 	deps            []packageID
 	missingDeps     map[packagePath]struct{}
 	module          *packages.Module
+	depsErrors      []*packagesinternal.PackageError
 
 	// config is the *packages.Config associated with the loaded package.
 	config *packages.Config
@@ -203,11 +204,11 @@ func (s *snapshot) workspaceLayoutError(ctx context.Context) *source.CriticalErr
 	if !s.ValidBuildConfiguration() {
 		msg := `gopls requires a module at the root of your workspace.
 You can work with multiple modules by opening each one as a workspace folder.
-Improvements to this workflow will be coming soon (https://github.com/golang/go/issues/32394),
-and you can learn more here: https://github.com/golang/go/issues/36899.`
+Improvements to this workflow will be coming soon, and you can learn more here:
+https://github.com/golang/tools/blob/master/gopls/doc/workspace.md.`
 		return &source.CriticalError{
 			MainError: errors.Errorf(msg),
-			ErrorList: s.applyCriticalErrorToFiles(ctx, msg, openFiles),
+			DiagList:  s.applyCriticalErrorToFiles(ctx, msg, openFiles),
 		}
 	}
 
@@ -220,37 +221,40 @@ and you can learn more here: https://github.com/golang/go/issues/36899.`
 		for uri := range s.workspace.getActiveModFiles() {
 			rootModURI = uri
 		}
-		nestedModules := map[span.URI][]source.VersionedFileHandle{}
+		nestedModules := map[string][]source.VersionedFileHandle{}
 		for _, fh := range openFiles {
 			modURI := moduleForURI(s.workspace.knownModFiles, fh.URI())
 			if modURI != rootModURI {
-				nestedModules[modURI] = append(nestedModules[modURI], fh)
+				modDir := filepath.Dir(modURI.Filename())
+				nestedModules[modDir] = append(nestedModules[modDir], fh)
 			}
 		}
 		// Add a diagnostic to each file in a nested module to mark it as
 		// "orphaned". Don't show a general diagnostic in the progress bar,
 		// because the user may still want to edit a file in a nested module.
-		var srcErrs []*source.Error
-		for modURI, uris := range nestedModules {
+		var srcDiags []*source.Diagnostic
+		for modDir, uris := range nestedModules {
 			msg := fmt.Sprintf(`This file is in %s, which is a nested module in the %s module.
 gopls currently requires one module per workspace folder.
 Please open %s as a separate workspace folder.
-You can learn more here: https://github.com/golang/go/issues/36899.
-`, modURI.Filename(), rootModURI.Filename(), modURI.Filename())
-			srcErrs = append(srcErrs, s.applyCriticalErrorToFiles(ctx, msg, uris)...)
+You can learn more here: https://github.com/golang/tools/blob/master/gopls/doc/workspace.md.
+`, modDir, filepath.Dir(rootModURI.Filename()), modDir)
+			srcDiags = append(srcDiags, s.applyCriticalErrorToFiles(ctx, msg, uris)...)
 		}
-		if len(srcErrs) != 0 {
+		if len(srcDiags) != 0 {
 			return &source.CriticalError{
-				MainError: errors.Errorf(`You are working in a nested module. Please open it as a separate workspace folder.`),
-				ErrorList: srcErrs,
+				MainError: errors.Errorf(`You are working in a nested module.
+Please open it as a separate workspace folder. Learn more:
+https://github.com/golang/tools/blob/master/gopls/doc/workspace.md.`),
+				DiagList: srcDiags,
 			}
 		}
 	}
 	return nil
 }
 
-func (s *snapshot) applyCriticalErrorToFiles(ctx context.Context, msg string, files []source.VersionedFileHandle) []*source.Error {
-	var srcErrs []*source.Error
+func (s *snapshot) applyCriticalErrorToFiles(ctx context.Context, msg string, files []source.VersionedFileHandle) []*source.Diagnostic {
+	var srcDiags []*source.Diagnostic
 	for _, fh := range files {
 		// Place the diagnostics on the package or module declarations.
 		var rng protocol.Range
@@ -269,14 +273,15 @@ func (s *snapshot) applyCriticalErrorToFiles(ctx context.Context, msg string, fi
 				}
 			}
 		}
-		srcErrs = append(srcErrs, &source.Error{
-			URI:     fh.URI(),
-			Range:   rng,
-			Kind:    source.ListError,
-			Message: msg,
+		srcDiags = append(srcDiags, &source.Diagnostic{
+			URI:      fh.URI(),
+			Range:    rng,
+			Severity: protocol.SeverityError,
+			Source:   source.ListError,
+			Message:  msg,
 		})
 	}
-	return srcErrs
+	return srcDiags
 }
 
 type workspaceDirKey string
@@ -370,6 +375,7 @@ func (s *snapshot) setMetadata(ctx context.Context, pkgPath packagePath, pkg *pa
 		errors:     pkg.Errors,
 		config:     cfg,
 		module:     pkg.Module,
+		depsErrors: packagesinternal.GetDepsErrors(pkg),
 	}
 
 	for _, filename := range pkg.CompiledGoFiles {
