@@ -12,7 +12,6 @@ import (
 	"io"
 	"io/ioutil"
 	"os"
-	"os/exec"
 	"path"
 	"path/filepath"
 	"reflect"
@@ -23,9 +22,12 @@ import (
 
 	"golang.org/x/mod/modfile"
 	"golang.org/x/mod/semver"
+	exec "golang.org/x/sys/execabs"
+	"golang.org/x/tools/go/packages"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/gocommand"
 	"golang.org/x/tools/internal/imports"
+	"golang.org/x/tools/internal/lsp/protocol"
 	"golang.org/x/tools/internal/lsp/source"
 	"golang.org/x/tools/internal/memoize"
 	"golang.org/x/tools/internal/span"
@@ -47,10 +49,6 @@ type View struct {
 	// background contexts created for this view.
 	baseCtx context.Context
 
-	// backgroundCtx is the current context used by background tasks initiated
-	// by the view.
-	backgroundCtx context.Context
-
 	// cancel is called when all action being performed by the current view
 	// should be stopped.
 	cancel context.CancelFunc
@@ -63,6 +61,9 @@ type View struct {
 
 	importsState *importsState
 
+	// moduleUpgrades tracks known upgrades for module paths.
+	moduleUpgrades map[string]string
+
 	// keep track of files by uri and by basename, a single file may be mapped
 	// to multiple uris, and the same basename may map to multiple files
 	filesByURI  map[span.URI]*fileBase
@@ -110,16 +111,27 @@ type workspaceInformation struct {
 	// `go env` variables that need to be tracked by gopls.
 	environmentVariables
 
+	// userGo111Module is the user's value of GO111MODULE.
+	userGo111Module go111module
+
 	// The value of GO111MODULE we want to run with.
-	go111module string
+	effectiveGo111Module string
 
 	// goEnv is the `go env` output collected when a view is created.
 	// It includes the values of the environment variables above.
 	goEnv map[string]string
 }
 
+type go111module int
+
+const (
+	off = go111module(iota)
+	auto
+	on
+)
+
 type environmentVariables struct {
-	gocache, gopath, goprivate, gomodcache string
+	gocache, gopath, goroot, goprivate, gomodcache, go111module string
 }
 
 type workspaceMode int
@@ -235,6 +247,9 @@ func minorOptionsChange(a, b *source.Options) bool {
 	if !reflect.DeepEqual(a.Env, b.Env) {
 		return false
 	}
+	if !reflect.DeepEqual(a.DirectoryFilters, b.DirectoryFilters) {
+		return false
+	}
 	aBuildFlags := make([]string, len(a.BuildFlags))
 	bBuildFlags := make([]string, len(b.BuildFlags))
 	copy(aBuildFlags, a.BuildFlags)
@@ -316,7 +331,16 @@ func (s *snapshot) RunProcessEnvFunc(ctx context.Context, fn func(*imports.Optio
 }
 
 func (v *View) contains(uri span.URI) bool {
-	return source.InDir(v.rootURI.Filename(), uri.Filename()) || source.InDir(v.folder.Filename(), uri.Filename())
+	inRoot := source.InDir(v.rootURI.Filename(), uri.Filename())
+	inFolder := source.InDir(v.folder.Filename(), uri.Filename())
+	if !inRoot && !inFolder {
+		return false
+	}
+	// Filters are applied relative to the workspace folder.
+	if inFolder {
+		return !pathExcludedByFilter(strings.TrimPrefix(uri.Filename(), v.folder.Filename()), v.Options())
+	}
+	return true
 }
 
 func (v *View) mapFile(uri span.URI, f *fileBase) {
@@ -333,15 +357,23 @@ func basename(filename string) string {
 
 func (v *View) relevantChange(c source.FileModification) bool {
 	// If the file is known to the view, the change is relevant.
-	known := v.knownFile(c.URI)
-
+	if v.knownFile(c.URI) {
+		return true
+	}
+	// The gopls.mod may not be "known" because we first access it through the
+	// session. As a result, treat changes to the view's gopls.mod file as
+	// always relevant, even if they are only on-disk changes.
+	// TODO(rstambler): Make sure the gopls.mod is always known to the view.
+	if c.URI == goplsModURI(v.rootURI) {
+		return true
+	}
 	// If the file is not known to the view, and the change is only on-disk,
 	// we should not invalidate the snapshot. This is necessary because Emacs
 	// sends didChangeWatchedFiles events for temp files.
-	if !known && c.OnDisk && (c.Action == source.Change || c.Action == source.Delete) {
+	if c.OnDisk && (c.Action == source.Change || c.Action == source.Delete) {
 		return false
 	}
-	return v.contains(c.URI) || known
+	return v.contains(c.URI)
 }
 
 func (v *View) knownFile(uri span.URI) bool {
@@ -352,24 +384,21 @@ func (v *View) knownFile(uri span.URI) bool {
 	return f != nil && err == nil
 }
 
-// getFile returns a file for the given URI. It will always succeed because it
-// adds the file to the managed set if needed.
-func (v *View) getFile(uri span.URI) (*fileBase, error) {
+// getFile returns a file for the given URI.
+func (v *View) getFile(uri span.URI) *fileBase {
 	v.mu.Lock()
 	defer v.mu.Unlock()
 
-	f, err := v.findFile(uri)
-	if err != nil {
-		return nil, err
-	} else if f != nil {
-		return f, nil
+	f, _ := v.findFile(uri)
+	if f != nil {
+		return f
 	}
 	f = &fileBase{
 		view:  v,
 		fname: uri.Filename(),
 	}
 	v.mapFile(uri, f)
-	return f, nil
+	return f
 }
 
 // findFile checks the cache for any file matching the given uri.
@@ -438,23 +467,16 @@ func (v *View) Session() *Session {
 	return v.session
 }
 
-func (v *View) BackgroundContext() context.Context {
-	v.mu.Lock()
-	defer v.mu.Unlock()
-
-	return v.backgroundCtx
-}
-
 func (s *snapshot) IgnoredFile(uri span.URI) bool {
 	filename := uri.Filename()
 	var prefixes []string
-	if len(s.workspace.activeModFiles()) == 0 {
+	if len(s.workspace.getActiveModFiles()) == 0 {
 		for _, entry := range filepath.SplitList(s.view.gopath) {
 			prefixes = append(prefixes, filepath.Join(entry, "src"))
 		}
 	} else {
 		prefixes = append(prefixes, s.view.gomodcache)
-		for m := range s.workspace.activeModFiles() {
+		for m := range s.workspace.getActiveModFiles() {
 			prefixes = append(prefixes, dirURI(m).Filename())
 		}
 	}
@@ -515,19 +537,16 @@ func (s *snapshot) initialize(ctx context.Context, firstAttempt bool) {
 
 		// If we have multiple modules, we need to load them by paths.
 		var scopes []interface{}
-		var modErrors *source.ErrorList
+		var modDiagnostics []*source.Diagnostic
 		addError := func(uri span.URI, err error) {
-			if modErrors == nil {
-				modErrors = &source.ErrorList{}
-			}
-			*modErrors = append(*modErrors, &source.Error{
+			modDiagnostics = append(modDiagnostics, &source.Diagnostic{
 				URI:      uri,
-				Category: "compiler",
-				Kind:     source.ListError,
+				Severity: protocol.SeverityError,
+				Source:   source.ListError,
 				Message:  err.Error(),
 			})
 		}
-		for modURI := range s.workspace.activeModFiles() {
+		for modURI := range s.workspace.getActiveModFiles() {
 			fh, err := s.GetFile(ctx, modURI)
 			if err != nil {
 				addError(modURI, err)
@@ -548,30 +567,39 @@ func (s *snapshot) initialize(ctx context.Context, firstAttempt bool) {
 		if len(scopes) == 0 {
 			scopes = append(scopes, viewLoadScope("LOAD_VIEW"))
 		}
-		err := s.load(ctx, append(scopes, packagePath("builtin"))...)
+		err := s.load(ctx, firstAttempt, append(scopes, packagePath("builtin"))...)
 		if ctx.Err() != nil {
 			return
 		}
 		if err != nil {
 			event.Error(ctx, "initial workspace load failed", err)
-			if modErrors != nil {
-				s.initializedErr = errors.Errorf("errors loading modules: %v: %w", err, modErrors)
-			} else {
-				s.initializedErr = err
+			extractedDiags, _ := s.extractGoCommandErrors(ctx, err.Error())
+			s.initializedErr = &source.CriticalError{
+				MainError: err,
+				DiagList:  append(modDiagnostics, extractedDiags...),
 			}
+		} else if len(modDiagnostics) != 0 {
+			s.initializedErr = &source.CriticalError{
+				MainError: fmt.Errorf("error loading module names"),
+				DiagList:  modDiagnostics,
+			}
+		} else {
+			// Clear out the initialization error, in case it had been set
+			// previously.
+			s.initializedErr = nil
 		}
 	})
 }
 
 // invalidateContent invalidates the content of a Go file,
 // including any position and type information that depends on it.
-func (v *View) invalidateContent(ctx context.Context, changes map[span.URI]*fileChange, forceReloadMetadata bool) (source.Snapshot, func()) {
+func (v *View) invalidateContent(ctx context.Context, changes map[span.URI]*fileChange, forceReloadMetadata bool) (*snapshot, func()) {
 	// Detach the context so that content invalidation cannot be canceled.
 	ctx = xcontext.Detach(ctx)
 
 	// Cancel all still-running previous requests, since they would be
 	// operating on stale data.
-	v.cancelBackground()
+	v.snapshot.cancel()
 
 	// Do not clone a snapshot until its view has finished initializing.
 	v.snapshot.AwaitInitialized(ctx)
@@ -583,7 +611,7 @@ func (v *View) invalidateContent(ctx context.Context, changes map[span.URI]*file
 	oldSnapshot := v.snapshot
 
 	var workspaceChanged bool
-	v.snapshot, workspaceChanged = oldSnapshot.clone(ctx, changes, forceReloadMetadata)
+	v.snapshot, workspaceChanged = oldSnapshot.clone(ctx, v.baseCtx, changes, forceReloadMetadata)
 	if workspaceChanged && v.tempWorkspace != "" {
 		snap := v.snapshot
 		go func() {
@@ -602,35 +630,26 @@ func (v *View) invalidateContent(ctx context.Context, changes map[span.URI]*file
 }
 
 func copyWorkspace(dst span.URI, src span.URI) error {
-	srcMod := filepath.Join(src.Filename(), "go.mod")
-	srcf, err := os.Open(srcMod)
-	if err != nil {
-		return errors.Errorf("opening snapshot mod file: %w", err)
-	}
-	defer srcf.Close()
-	dstMod := filepath.Join(dst.Filename(), "go.mod")
-	dstf, err := os.Create(dstMod)
-	if err != nil {
-		return errors.Errorf("truncating view mod file: %w", err)
-	}
-	defer dstf.Close()
-	if _, err := io.Copy(dstf, srcf); err != nil {
-		return errors.Errorf("copying modfiles: %w", err)
+	for _, name := range []string{"go.mod", "go.sum"} {
+		srcname := filepath.Join(src.Filename(), name)
+		srcf, err := os.Open(srcname)
+		if err != nil {
+			return errors.Errorf("opening snapshot %s: %w", name, err)
+		}
+		defer srcf.Close()
+		dstname := filepath.Join(dst.Filename(), name)
+		dstf, err := os.Create(dstname)
+		if err != nil {
+			return errors.Errorf("truncating view %s: %w", name, err)
+		}
+		defer dstf.Close()
+		if _, err := io.Copy(dstf, srcf); err != nil {
+			return errors.Errorf("copying %s: %w", name, err)
+		}
 	}
 	return nil
 }
 
-func (v *View) cancelBackground() {
-	v.mu.Lock()
-	defer v.mu.Unlock()
-	if v.cancel == nil {
-		// this can happen during shutdown
-		return
-	}
-	v.cancel()
-	v.backgroundCtx, v.cancel = context.WithCancel(v.baseCtx)
-}
-
 func (s *Session) getWorkspaceInformation(ctx context.Context, folder span.URI, options *source.Options) (*workspaceInformation, error) {
 	if err := checkPathCase(folder.Filename()); err != nil {
 		return nil, errors.Errorf("invalid workspace configuration: %w", err)
@@ -649,17 +668,16 @@ func (s *Session) getWorkspaceInformation(ctx context.Context, folder span.URI,
 	if v, ok := options.Env["GO111MODULE"]; ok {
 		go111module = v
 	}
+	// Make sure to get the `go env` before continuing with initialization.
+	envVars, env, err := s.getGoEnv(ctx, folder.Filename(), goversion, go111module, options.EnvSlice())
+	if err != nil {
+		return nil, err
+	}
 	// If using 1.16, change the default back to auto. The primary effect of
 	// GO111MODULE=on is to break GOPATH, which we aren't too interested in.
 	if goversion >= 16 && go111module == "" {
 		go111module = "auto"
 	}
-
-	// Make sure to get the `go env` before continuing with initialization.
-	envVars, env, err := s.getGoEnv(ctx, folder.Filename(), append(options.EnvSlice(), "GO111MODULE="+go111module))
-	if err != nil {
-		return nil, err
-	}
 	// The value of GOPACKAGESDRIVER is not returned through the go command.
 	gopackagesdriver := os.Getenv("GOPACKAGESDRIVER")
 	for _, s := range env {
@@ -668,6 +686,7 @@ func (s *Session) getWorkspaceInformation(ctx context.Context, folder span.URI,
 			gopackagesdriver = split[1]
 		}
 	}
+
 	// A user may also have a gopackagesdriver binary on their machine, which
 	// works the same way as setting GOPACKAGESDRIVER.
 	tool, _ := exec.LookPath("gopackagesdriver")
@@ -675,13 +694,26 @@ func (s *Session) getWorkspaceInformation(ctx context.Context, folder span.URI,
 
 	return &workspaceInformation{
 		hasGopackagesDriver:  hasGopackagesDriver,
-		go111module:          go111module,
+		effectiveGo111Module: go111module,
+		userGo111Module:      go111moduleForVersion(go111module, goversion),
 		goversion:            goversion,
 		environmentVariables: envVars,
 		goEnv:                env,
 	}, nil
 }
 
+func go111moduleForVersion(go111module string, goversion int) go111module {
+	// Off by default until Go 1.12.
+	if go111module == "off" || (goversion < 12 && go111module == "") {
+		return off
+	}
+	// On by default as of Go 1.16.
+	if go111module == "on" || (goversion >= 16 && go111module == "") {
+		return on
+	}
+	return auto
+}
+
 // findWorkspaceRoot searches for the best workspace root according to the
 // following heuristics:
 //   - First, look for a parent directory containing a gopls.mod file
@@ -692,7 +724,7 @@ func (s *Session) getWorkspaceInformation(ctx context.Context, folder span.URI,
 // Otherwise, it returns folder.
 // TODO (rFindley): move this to workspace.go
 // TODO (rFindley): simplify this once workspace modules are enabled by default.
-func findWorkspaceRoot(ctx context.Context, folder span.URI, fs source.FileSource, experimental bool) (span.URI, error) {
+func findWorkspaceRoot(ctx context.Context, folder span.URI, fs source.FileSource, excludePath func(string) bool, experimental bool) (span.URI, error) {
 	patterns := []string{"go.mod"}
 	if experimental {
 		patterns = []string{"gopls.mod", "go.mod"}
@@ -713,12 +745,11 @@ func findWorkspaceRoot(ctx context.Context, folder span.URI, fs source.FileSourc
 	}
 
 	// ...else we should check if there's exactly one nested module.
-	const filesToSearch = 10000
-	all, err := findModules(ctx, folder, 2, filesToSearch)
+	all, err := findModules(ctx, folder, excludePath, 2)
 	if err == errExhausted {
 		// Fall-back behavior: if we don't find any modules after searching 10000
 		// files, assume there are none.
-		event.Log(ctx, fmt.Sprintf("stopped searching for modules after %d files", filesToSearch))
+		event.Log(ctx, fmt.Sprintf("stopped searching for modules after %d files", fileLimit))
 		return folder, nil
 	}
 	if err != nil {
@@ -782,14 +813,17 @@ func validBuildConfiguration(folder span.URI, ws *workspaceInformation, modFiles
 }
 
 // getGoEnv gets the view's various GO* values.
-func (s *Session) getGoEnv(ctx context.Context, folder string, configEnv []string) (environmentVariables, map[string]string, error) {
+func (s *Session) getGoEnv(ctx context.Context, folder string, goversion int, go111module string, configEnv []string) (environmentVariables, map[string]string, error) {
 	envVars := environmentVariables{}
 	vars := map[string]*string{
-		"GOCACHE":    &envVars.gocache,
-		"GOPATH":     &envVars.gopath,
-		"GOPRIVATE":  &envVars.goprivate,
-		"GOMODCACHE": &envVars.gomodcache,
+		"GOCACHE":     &envVars.gocache,
+		"GOPATH":      &envVars.gopath,
+		"GOROOT":      &envVars.goroot,
+		"GOPRIVATE":   &envVars.goprivate,
+		"GOMODCACHE":  &envVars.gomodcache,
+		"GO111MODULE": &envVars.go111module,
 	}
+
 	// We can save ~200 ms by requesting only the variables we care about.
 	args := append([]string{"-json"}, imports.RequiredGoEnvVars...)
 	for k := range vars {
@@ -821,6 +855,10 @@ func (s *Session) getGoEnv(ctx context.Context, folder string, configEnv []strin
 	if envVars.gomodcache == "" && envVars.gopath != "" {
 		envVars.gomodcache = filepath.Join(filepath.SplitList(envVars.gopath)[0], "pkg/mod")
 	}
+	// GO111MODULE does not appear in `go env` output until Go 1.13.
+	if goversion < 13 {
+		envVars.go111module = go111module
+	}
 	return envVars, env, err
 }
 
@@ -828,6 +866,26 @@ func (v *View) IsGoPrivatePath(target string) bool {
 	return globsMatchPath(v.goprivate, target)
 }
 
+func (v *View) ModuleUpgrades() map[string]string {
+	v.mu.Lock()
+	defer v.mu.Unlock()
+
+	upgrades := map[string]string{}
+	for mod, ver := range v.moduleUpgrades {
+		upgrades[mod] = ver
+	}
+	return upgrades
+}
+
+func (v *View) RegisterModuleUpgrades(upgrades map[string]string) {
+	v.mu.Lock()
+	defer v.mu.Unlock()
+
+	for mod, ver := range upgrades {
+		v.moduleUpgrades[mod] = ver
+	}
+}
+
 // Copied from
 // https://cs.opensource.google/go/go/+/master:src/cmd/go/internal/str/path.go;l=58;drc=2910c5b4a01a573ebc97744890a07c1a3122c67a
 func globsMatchPath(globs, target string) bool {
@@ -876,11 +934,10 @@ var modFlagRegexp = regexp.MustCompile(`-mod[ =](\w+)`)
 // after we have a version of the workspace go.mod file on disk. Getting a
 // FileHandle from the cache for temporary files is problematic, since we
 // cannot delete it.
-func (s *snapshot) needsModEqualsMod(ctx context.Context, modURI span.URI, modContent []byte) (bool, error) {
-	if s.view.goversion < 16 || s.workspaceMode()&moduleMode == 0 {
+func (s *snapshot) vendorEnabled(ctx context.Context, modURI span.URI, modContent []byte) (bool, error) {
+	if s.workspaceMode()&moduleMode == 0 {
 		return false, nil
 	}
-
 	matches := modFlagRegexp.FindStringSubmatch(s.view.goEnv["GOFLAGS"])
 	var modFlag string
 	if len(matches) != 0 {
@@ -898,8 +955,48 @@ func (s *snapshot) needsModEqualsMod(ctx context.Context, modURI span.URI, modCo
 		return false, err
 	}
 	if fi, err := os.Stat(filepath.Join(s.view.rootURI.Filename(), "vendor")); err != nil || !fi.IsDir() {
-		return true, nil
+		return false, nil
 	}
 	vendorEnabled := modFile.Go != nil && modFile.Go.Version != "" && semver.Compare("v"+modFile.Go.Version, "v1.14") >= 0
-	return !vendorEnabled, nil
+	return vendorEnabled, nil
+}
+
+func (v *View) allFilesExcluded(pkg *packages.Package) bool {
+	opts := v.Options()
+	folder := filepath.ToSlash(v.folder.Filename())
+	for _, f := range pkg.GoFiles {
+		f = filepath.ToSlash(f)
+		if !strings.HasPrefix(f, folder) {
+			return false
+		}
+		if !pathExcludedByFilter(strings.TrimPrefix(f, folder), opts) {
+			return false
+		}
+	}
+	return true
+}
+
+func pathExcludedByFilterFunc(opts *source.Options) func(string) bool {
+	return func(path string) bool {
+		return pathExcludedByFilter(path, opts)
+	}
+}
+
+func pathExcludedByFilter(path string, opts *source.Options) bool {
+	path = strings.TrimPrefix(filepath.ToSlash(path), "/")
+
+	excluded := false
+	for _, filter := range opts.DirectoryFilters {
+		op, prefix := filter[0], filter[1:]
+		// Non-empty prefixes have to be precise directory matches.
+		if prefix != "" {
+			prefix = prefix + "/"
+			path = path + "/"
+		}
+		if !strings.HasPrefix(path, prefix) {
+			continue
+		}
+		excluded = op == '-'
+	}
+	return excluded
 }
