@@ -40,6 +40,7 @@ type runner struct {
 	diagnostics map[span.URI][]*source.Diagnostic
 	ctx         context.Context
 	normalizers []tests.Normalizer
+	editRecv    chan map[span.URI]string
 }
 
 func testLSP(t *testing.T, datum *tests.Data) {
@@ -85,17 +86,19 @@ func testLSP(t *testing.T, datum *tests.Data) {
 		t.Fatal(err)
 	}
 	r := &runner{
-		server:      NewServer(session, testClient{}),
 		data:        datum,
 		ctx:         ctx,
 		normalizers: tests.CollectNormalizers(datum.Exported),
+		editRecv:    make(chan map[span.URI]string, 1),
 	}
+	r.server = NewServer(session, testClient{runner: r})
 	tests.Run(t, r, datum)
 }
 
 // testClient stubs any client functions that may be called by LSP functions.
 type testClient struct {
 	protocol.Client
+	runner *runner
 }
 
 // Trivially implement PublishDiagnostics so that we can call
@@ -104,6 +107,15 @@ func (c testClient) PublishDiagnostics(context.Context, *protocol.PublishDiagnos
 	return nil
 }
 
+func (c testClient) ApplyEdit(ctx context.Context, params *protocol.ApplyWorkspaceEditParams) (*protocol.ApplyWorkspaceEditResponse, error) {
+	res, err := applyTextDocumentEdits(c.runner, params.Edit.DocumentChanges)
+	if err != nil {
+		return nil, err
+	}
+	c.runner.editRecv <- res
+	return &protocol.ApplyWorkspaceEditResponse{Applied: true}, nil
+}
+
 func (r *runner) CallHierarchy(t *testing.T, spn span.Span, expectedCalls *tests.CallHierarchyResult) {
 	mapper, err := r.data.Mapper(spn.URI())
 	if err != nil {
@@ -453,7 +465,10 @@ func (r *runner) Import(t *testing.T, spn span.Span) {
 		return []byte(got), nil
 	}))
 	if want != got {
-		d := myers.ComputeEdits(uri, want, got)
+		d, err := myers.ComputeEdits(uri, want, got)
+		if err != nil {
+			t.Fatal(err)
+		}
 		t.Errorf("import failed for %s: %s", filename, diff.ToUnified("want", "got", want, d))
 	}
 }
@@ -465,13 +480,6 @@ func (r *runner) SuggestedFix(t *testing.T, spn span.Span, actionKinds []string,
 		t.Fatal(err)
 	}
 
-	snapshot, release := view.Snapshot(r.ctx)
-	defer release()
-
-	fh, err := snapshot.GetVersionedFile(r.ctx, uri)
-	if err != nil {
-		t.Fatal(err)
-	}
 	m, err := r.data.Mapper(uri)
 	if err != nil {
 		t.Fatal(err)
@@ -530,14 +538,14 @@ func (r *runner) SuggestedFix(t *testing.T, spn span.Span, actionKinds []string,
 	}
 	var res map[span.URI]string
 	if cmd := action.Command; cmd != nil {
-		edits, err := commandToEdits(r.ctx, snapshot, fh, rng, action.Command.Command)
+		_, err := r.server.ExecuteCommand(r.ctx, &protocol.ExecuteCommandParams{
+			Command:   action.Command.Command,
+			Arguments: action.Command.Arguments,
+		})
 		if err != nil {
 			t.Fatalf("error converting command %q to edits: %v", action.Command.Command, err)
 		}
-		res, err = applyTextDocumentEdits(r, edits)
-		if err != nil {
-			t.Fatal(err)
-		}
+		res = <-r.editRecv
 	} else {
 		res, err = applyTextDocumentEdits(r, action.Edit.DocumentChanges)
 		if err != nil {
@@ -549,46 +557,13 @@ func (r *runner) SuggestedFix(t *testing.T, spn span.Span, actionKinds []string,
 			return []byte(got), nil
 		}))
 		if want != got {
-			t.Errorf("suggested fixes failed for %s:\n%s", u.Filename(), tests.Diff(want, got))
+			t.Errorf("suggested fixes failed for %s:\n%s", u.Filename(), tests.Diff(t, want, got))
 		}
 	}
 }
 
-func commandToEdits(ctx context.Context, snapshot source.Snapshot, fh source.VersionedFileHandle, rng protocol.Range, cmd string) ([]protocol.TextDocumentEdit, error) {
-	var command *source.Command
-	for _, c := range source.Commands {
-		if c.ID() == cmd {
-			command = c
-			break
-		}
-	}
-	if command == nil {
-		return nil, fmt.Errorf("no known command for %s", cmd)
-	}
-	if !command.Applies(ctx, snapshot, fh, rng) {
-		return nil, fmt.Errorf("cannot apply %v", command.ID())
-	}
-	edits, err := command.SuggestedFix(ctx, snapshot, fh, rng)
-	if err != nil {
-		return nil, fmt.Errorf("error calling command.SuggestedFix: %v", err)
-	}
-	return edits, nil
-}
-
 func (r *runner) FunctionExtraction(t *testing.T, start span.Span, end span.Span) {
 	uri := start.URI()
-	view, err := r.server.session.ViewOf(uri)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	snapshot, release := view.Snapshot(r.ctx)
-	defer release()
-
-	fh, err := snapshot.GetVersionedFile(r.ctx, uri)
-	if err != nil {
-		t.Fatal(err)
-	}
 	m, err := r.data.Mapper(uri)
 	if err != nil {
 		t.Fatal(err)
@@ -615,20 +590,20 @@ func (r *runner) FunctionExtraction(t *testing.T, start span.Span, end span.Span
 	if len(actions) == 0 || len(actions) > 1 {
 		t.Fatalf("unexpected number of code actions, want 1, got %v", len(actions))
 	}
-	edits, err := commandToEdits(r.ctx, snapshot, fh, rng, actions[0].Command.Command)
-	if err != nil {
-		t.Fatal(err)
-	}
-	res, err := applyTextDocumentEdits(r, edits)
+	_, err = r.server.ExecuteCommand(r.ctx, &protocol.ExecuteCommandParams{
+		Command:   actions[0].Command.Command,
+		Arguments: actions[0].Command.Arguments,
+	})
 	if err != nil {
 		t.Fatal(err)
 	}
+	res := <-r.editRecv
 	for u, got := range res {
 		want := string(r.data.Golden("functionextraction_"+tests.SpanName(spn), u.Filename(), func() ([]byte, error) {
 			return []byte(got), nil
 		}))
 		if want != got {
-			t.Errorf("function extraction failed for %s:\n%s", u.Filename(), tests.Diff(want, got))
+			t.Errorf("function extraction failed for %s:\n%s", u.Filename(), tests.Diff(t, want, got))
 		}
 	}
 }
@@ -680,7 +655,7 @@ func (r *runner) Definition(t *testing.T, spn span.Span, d tests.Definition) {
 			return []byte(hover.Contents.Value), nil
 		}))
 		if hover.Contents.Value != expectHover {
-			t.Errorf("%s:\n%s", d.Src, tests.Diff(expectHover, hover.Contents.Value))
+			t.Errorf("%s:\n%s", d.Src, tests.Diff(t, expectHover, hover.Contents.Value))
 		}
 	}
 	if !d.OnlyHover {
@@ -905,7 +880,7 @@ func (r *runner) Rename(t *testing.T, spn span.Span, newText string) {
 		return []byte(got), nil
 	}))
 	if want != got {
-		t.Errorf("rename failed for %s:\n%s", newText, tests.Diff(want, got))
+		t.Errorf("rename failed for %s:\n%s", newText, tests.Diff(t, want, got))
 	}
 }
 
@@ -1052,7 +1027,7 @@ func (r *runner) callWorkspaceSymbols(t *testing.T, uri span.URI, query string,
 	want := string(r.data.Golden(fmt.Sprintf("workspace_symbol-%s-%s", strings.ToLower(string(matcher)), query), uri.Filename(), func() ([]byte, error) {
 		return []byte(got), nil
 	}))
-	if diff := tests.Diff(want, got); diff != "" {
+	if diff := tests.Diff(t, want, got); diff != "" {
 		t.Error(diff)
 	}
 }
@@ -1092,7 +1067,11 @@ func (r *runner) SignatureHelp(t *testing.T, spn span.Span, want *protocol.Signa
 	if got == nil {
 		t.Fatalf("expected %v, got nil", want)
 	}
-	if diff := tests.DiffSignatures(spn, want, got); diff != "" {
+	diff, err := tests.DiffSignatures(spn, want, got)
+	if err != nil {
+		t.Fatal(err)
+	}
+	if diff != "" {
 		t.Error(diff)
 	}
 }
@@ -1171,19 +1150,10 @@ func (r *runner) collectDiagnostics(view source.View) {
 	// Always run diagnostics with analysis.
 	r.server.diagnose(r.ctx, snapshot, true)
 	for uri, reports := range r.server.diagnostics {
-		var diagnostics []*source.Diagnostic
 		for _, report := range reports.reports {
 			for _, d := range report.diags {
-				diagnostics = append(diagnostics, &source.Diagnostic{
-					Range:    d.Range,
-					Message:  d.Message,
-					Related:  d.Related,
-					Severity: d.Severity,
-					Source:   d.Source,
-					Tags:     d.Tags,
-				})
+				r.diagnostics[uri] = append(r.diagnostics[uri], d)
 			}
-			r.diagnostics[uri] = diagnostics
 		}
 	}
 }
