@@ -43,7 +43,7 @@ func (s *Server) semanticTokensRefresh(ctx context.Context) error {
 
 func (s *Server) computeSemanticTokens(ctx context.Context, td protocol.TextDocumentIdentifier, rng *protocol.Range) (*protocol.SemanticTokens, error) {
 	ans := protocol.SemanticTokens{
-		Data: []float64{},
+		Data: []uint32{},
 	}
 	snapshot, _, ok, release, err := s.beginFileRequest(ctx, td.URI, source.Go)
 	defer release()
@@ -157,15 +157,15 @@ func (e *encoded) token(start token.Pos, leng int, typ tokenType, mods []string)
 	e.add(lspRange.Start.Line, lspRange.Start.Character, length, typ, mods)
 }
 
-func (e *encoded) add(line, start float64, len float64, tok tokenType, mod []string) {
+func (e *encoded) add(line, start uint32, len uint32, tok tokenType, mod []string) {
 	x := semItem{line, start, len, tok, mod}
 	e.items = append(e.items, x)
 }
 
 // semItem represents a token found walking the parse tree
 type semItem struct {
-	line, start float64
-	len         float64
+	line, start uint32
+	len         uint32
 	typeStr     tokenType
 	mods        []string
 }
@@ -196,7 +196,7 @@ func (e *encoded) strStack() string {
 	if len(e.stack) > 0 {
 		loc := e.stack[len(e.stack)-1].Pos()
 		add := e.pgf.Tok.PositionFor(loc, false)
-		msg = append(msg, fmt.Sprintf("(%d:%d)", add.Line, add.Column))
+		msg = append(msg, fmt.Sprintf("(line:%d,col:%d)", add.Line, add.Column))
 	}
 	msg = append(msg, "]")
 	return strings.Join(msg, " ")
@@ -372,25 +372,8 @@ func (e *encoded) ident(x *ast.Ident) {
 	case *types.Const:
 		mods := []string{"readonly"}
 		tt := y.Type()
-		if ttx, ok := tt.(*types.Basic); ok {
-			switch bi := ttx.Info(); {
-			case bi&types.IsNumeric != 0:
-				me := tokVariable
-				if x.String() == "iota" {
-					me = tokKeyword
-					mods = []string{}
-				}
-				e.token(x.Pos(), len(x.String()), me, mods)
-			case bi&types.IsString != 0:
-				e.token(x.Pos(), len(x.String()), tokString, mods)
-			case bi&types.IsBoolean != 0:
-				e.token(x.Pos(), len(x.Name), tokKeyword, nil)
-			case bi == 0:
-				e.token(x.Pos(), len(x.String()), tokVariable, mods)
-			default:
-				msg := fmt.Sprintf("unexpected %x at %s", bi, e.pgf.Tok.PositionFor(x.Pos(), false))
-				e.unexpected(msg)
-			}
+		if _, ok := tt.(*types.Basic); ok {
+			e.token(x.Pos(), len(x.String()), tokVariable, mods)
 			break
 		}
 		if ttx, ok := tt.(*types.Named); ok {
@@ -411,7 +394,7 @@ func (e *encoded) ident(x *ast.Ident) {
 		// nothing to map it to
 	case *types.Nil:
 		// nil is a predeclared identifier
-		e.token(x.Pos(), len("nil"), tokKeyword, []string{"readonly"})
+		e.token(x.Pos(), len("nil"), tokVariable, []string{"readonly"})
 	case *types.PkgName:
 		e.token(x.Pos(), len(x.Name), tokNamespace, nil)
 	case *types.TypeName:
@@ -502,7 +485,7 @@ func (e *encoded) init() error {
 	return nil
 }
 
-func (e *encoded) Data() ([]float64, error) {
+func (e *encoded) Data() ([]uint32, error) {
 	// binary operators, at least, will be out of order
 	sort.Slice(e.items, func(i, j int) bool {
 		if e.items[i].line != e.items[j].line {
@@ -513,7 +496,7 @@ func (e *encoded) Data() ([]float64, error) {
 	typeMap, modMap := e.maps()
 	// each semantic token needs five values
 	// (see Integer Encoding for Tokens in the LSP spec)
-	x := make([]float64, 5*len(e.items))
+	x := make([]uint32, 5*len(e.items))
 	for i := 0; i < len(e.items); i++ {
 		j := 5 * i
 		if i == 0 {
@@ -526,12 +509,12 @@ func (e *encoded) Data() ([]float64, error) {
 			x[j+1] = e.items[i].start - e.items[i-1].start
 		}
 		x[j+2] = e.items[i].len
-		x[j+3] = float64(typeMap[e.items[i].typeStr])
+		x[j+3] = uint32(typeMap[e.items[i].typeStr])
 		mask := 0
 		for _, s := range e.items[i].mods {
 			mask |= modMap[s]
 		}
-		x[j+4] = float64(mask)
+		x[j+4] = uint32(mask)
 	}
 	return x, nil
 }
