@@ -388,6 +388,11 @@ type candidate struct {
 	// needs "..." appended.
 	variadic bool
 
+	// convertTo is a type that this candidate should be cast to. For
+	// example, if convertTo is float64, "foo" should be formatted as
+	// "float64(foo)".
+	convertTo types.Type
+
 	// imp is the import that needs to be added to this package in order
 	// for this candidate to be valid. nil if no import needed.
 	imp *importInfo
@@ -431,7 +436,6 @@ func Completion(ctx context.Context, snapshot source.Snapshot, fh source.FileHan
 		if innerErr != nil {
 			// return the error for GetParsedFile since it's more relevant in this situation.
 			return nil, nil, errors.Errorf("getting file for Completion: %w (package completions: %v)", err, innerErr)
-
 		}
 		return items, surrounding, nil
 	}
@@ -699,7 +703,7 @@ func (c *completer) sortItems() {
 		}
 
 		// Then sort by label so order stays consistent. This also has the
-		// effect of prefering shorter candidates.
+		// effect of preferring shorter candidates.
 		return c.items[i].Label < c.items[j].Label
 	})
 }
@@ -729,6 +733,10 @@ func (c *completer) emptySwitchStmt() bool {
 // (i.e. "golang.org/x/"). The user is meant to accept completion suggestions
 // until they reach a complete import path.
 func (c *completer) populateImportCompletions(ctx context.Context, searchImport *ast.ImportSpec) error {
+	if !strings.HasPrefix(searchImport.Path.Value, `"`) {
+		return nil
+	}
+
 	// deepSearch is not valuable for import completions.
 	c.deepState.enabled = false
 
@@ -1003,8 +1011,8 @@ func (c *completer) setSurroundingForComment(comments *ast.CommentGroup) {
 	c.setMatcherFromPrefix(c.surrounding.Prefix())
 }
 
-// isValidIdentifierChar returns true if a byte is a valid go identifier character
-// i.e unicode letter or digit or undescore
+// isValidIdentifierChar returns true if a byte is a valid go identifier
+// character, i.e. unicode letter or digit or underscore.
 func isValidIdentifierChar(char byte) bool {
 	charRune := rune(char)
 	return unicode.In(charRune, unicode.Letter, unicode.Digit) || char == '_'
@@ -1066,6 +1074,19 @@ func (c *completer) selector(ctx context.Context, sel *ast.SelectorExpr) error {
 	// Is sel a qualified identifier?
 	if id, ok := sel.X.(*ast.Ident); ok {
 		if pkgName, ok := c.pkg.GetTypesInfo().Uses[id].(*types.PkgName); ok {
+			var pkg source.Package
+			for _, imp := range c.pkg.Imports() {
+				if imp.PkgPath() == pkgName.Imported().Path() {
+					pkg = imp
+				}
+			}
+			// If the package is not imported, try searching for unimported
+			// completions.
+			if pkg == nil && c.opts.unimported {
+				if err := c.unimportedMembers(ctx, id); err != nil {
+					return err
+				}
+			}
 			candidates := c.packageMembers(pkgName.Imported(), stdScore, nil)
 			for _, cand := range candidates {
 				c.deepState.enqueue(cand)
@@ -1243,6 +1264,11 @@ func (c *completer) lexical(ctx context.Context) error {
 	var (
 		builtinIota = types.Universe.Lookup("iota")
 		builtinNil  = types.Universe.Lookup("nil")
+		// comparable is an interface that exists on the dev.typeparams Go branch.
+		// Filter it out from completion results to stabilize tests.
+		// TODO(rFindley) update (or remove) our handling for comparable once the
+		//                type parameter API has stabilized.
+		builtinComparable = types.Universe.Lookup("comparable")
 	)
 
 	// Track seen variables to avoid showing completions for shadowed variables.
@@ -1261,6 +1287,9 @@ func (c *completer) lexical(ctx context.Context) error {
 			if declScope != scope {
 				continue // Name was declared in some enclosing scope, or not at all.
 			}
+			if obj == builtinComparable {
+				continue
+			}
 
 			// If obj's type is invalid, find the AST node that defines the lexical block
 			// containing the declaration of obj. Don't resolve types for packages.
@@ -1280,12 +1309,10 @@ func (c *completer) lexical(ctx context.Context) error {
 				}
 			}
 
-			// Don't use LHS of value spec in RHS.
-			if vs := enclosingValueSpec(c.path); vs != nil {
-				for _, ident := range vs.Names {
-					if obj.Pos() == ident.Pos() {
-						continue Names
-					}
+			// Don't use LHS of decl in RHS.
+			for _, ident := range enclosingDeclLHS(c.path) {
+				if obj.Pos() == ident.Pos() {
+					continue Names
 				}
 			}
 
@@ -1671,12 +1698,12 @@ func (c *completer) expectedCompositeLiteralType() types.Type {
 	switch t := clInfo.clType.(type) {
 	case *types.Slice:
 		if clInfo.inKey {
-			return types.Typ[types.Int]
+			return types.Typ[types.UntypedInt]
 		}
 		return t.Elem()
 	case *types.Array:
 		if clInfo.inKey {
-			return types.Typ[types.Int]
+			return types.Typ[types.UntypedInt]
 		}
 		return t.Elem()
 	case *types.Map:
@@ -2019,7 +2046,7 @@ Nodes:
 		case *ast.SliceExpr:
 			// Make sure position falls within the brackets (e.g. "foo[a:<>]").
 			if node.Lbrack < c.pos && c.pos <= node.Rbrack {
-				inf.objType = types.Typ[types.Int]
+				inf.objType = types.Typ[types.UntypedInt]
 			}
 			return inf
 		case *ast.IndexExpr:
@@ -2030,7 +2057,7 @@ Nodes:
 					case *types.Map:
 						inf.objType = t.Key()
 					case *types.Slice, *types.Array:
-						inf.objType = types.Typ[types.Int]
+						inf.objType = types.Typ[types.UntypedInt]
 					}
 				}
 			}
@@ -2425,6 +2452,11 @@ func (c *completer) matchingCandidate(cand *candidate) bool {
 		return false
 	}
 
+	// Bail out early if we are completing a field name in a composite literal.
+	if v, ok := cand.obj.(*types.Var); ok && v.IsField() && c.wantStructFieldCompletions() {
+		return true
+	}
+
 	if isTypeName(cand.obj) {
 		return c.matchingTypeName(cand)
 	} else if c.wantTypeName() {
@@ -2490,8 +2522,18 @@ func (ci *candidateInference) candTypeMatches(cand *candidate) bool {
 				continue
 			}
 
-			matches, untyped := ci.typeMatches(expType, candType)
+			matches := ci.typeMatches(expType, candType)
 			if !matches {
+				// If candType doesn't otherwise match, consider if we can
+				// convert candType directly to expType.
+				if considerTypeConversion(candType, expType, cand.path) {
+					cand.convertTo = expType
+					// Give a major score penalty so we always prefer directly
+					// assignable candidates, all else equal.
+					cand.score *= 0.5
+					return true
+				}
+
 				continue
 			}
 
@@ -2502,8 +2544,15 @@ func (ci *candidateInference) candTypeMatches(cand *candidate) bool {
 			// Lower candidate score for untyped conversions. This avoids
 			// ranking untyped constants above candidates with an exact type
 			// match. Don't lower score of builtin constants, e.g. "true".
-			if untyped && !types.Identical(candType, expType) && cand.obj.Parent() != types.Universe {
-				cand.score /= 2
+			if isUntyped(candType) && !types.Identical(candType, expType) && cand.obj.Parent() != types.Universe {
+				// Bigger penalty for deep completions into other packages to
+				// avoid random constants from other packages popping up all
+				// the time.
+				if len(cand.path) > 0 && isPkgName(cand.path[0]) {
+					cand.score *= 0.5
+				} else {
+					cand.score *= 0.75
+				}
 			}
 
 			return true
@@ -2538,22 +2587,58 @@ func (ci *candidateInference) candTypeMatches(cand *candidate) bool {
 	})
 }
 
+// considerTypeConversion returns true if we should offer a completion
+// automatically converting "from" to "to".
+func considerTypeConversion(from, to types.Type, path []types.Object) bool {
+	// Don't offer to convert deep completions from other packages.
+	// Otherwise there are many random package level consts/vars that
+	// pop up as candidates all the time.
+	if len(path) > 0 && isPkgName(path[0]) {
+		return false
+	}
+
+	if !types.ConvertibleTo(from, to) {
+		return false
+	}
+
+	// Don't offer to convert ints to strings since that probably
+	// doesn't do what the user wants.
+	if isBasicKind(from, types.IsInteger) && isBasicKind(to, types.IsString) {
+		return false
+	}
+
+	return true
+}
+
 // typeMatches reports whether an object of candType makes a good
-// completion candidate given the expected type expType. It also
-// returns a second bool which is true if both types are basic types
-// of the same kind, and at least one is untyped.
-func (ci *candidateInference) typeMatches(expType, candType types.Type) (bool, bool) {
+// completion candidate given the expected type expType.
+func (ci *candidateInference) typeMatches(expType, candType types.Type) bool {
 	// Handle untyped values specially since AssignableTo gives false negatives
 	// for them (see https://golang.org/issue/32146).
 	if candBasic, ok := candType.Underlying().(*types.Basic); ok {
-		if wantBasic, ok := expType.Underlying().(*types.Basic); ok {
-			// Make sure at least one of them is untyped.
-			if isUntyped(candType) || isUntyped(expType) {
-				// Check that their constant kind (bool|int|float|complex|string) matches.
+		if expBasic, ok := expType.Underlying().(*types.Basic); ok {
+			// Note that the candidate and/or the expected can be untyped.
+			// In "fo<> == 100" the expected type is untyped, and the
+			// candidate could also be an untyped constant.
+
+			// Sort by is_untyped and then by is_int to simplify below logic.
+			a, b := candBasic.Info(), expBasic.Info()
+			if a&types.IsUntyped == 0 || (b&types.IsInteger > 0 && b&types.IsUntyped > 0) {
+				a, b = b, a
+			}
+
+			// If at least one is untyped...
+			if a&types.IsUntyped > 0 {
+				switch {
+				// Untyped integers are compatible with floats.
+				case a&types.IsInteger > 0 && b&types.IsFloat > 0:
+					return true
+
+				// Check if their constant kind (bool|int|float|complex|string) matches.
 				// This doesn't take into account the constant value, so there will be some
 				// false positives due to integer sign and overflow.
-				if candBasic.Info()&types.IsConstType == wantBasic.Info()&types.IsConstType {
-					return true, true
+				case a&types.IsConstType == b&types.IsConstType:
+					return true
 				}
 			}
 		}
@@ -2561,7 +2646,7 @@ func (ci *candidateInference) typeMatches(expType, candType types.Type) (bool, b
 
 	// AssignableTo covers the case where the types are equal, but also handles
 	// cases like assigning a concrete type to an interface type.
-	return types.AssignableTo(candType, expType), false
+	return types.AssignableTo(candType, expType)
 }
 
 // kindMatches reports whether candType's kind matches our expected
@@ -2622,7 +2707,7 @@ func (ci *candidateInference) assigneesMatch(cand *candidate, sig *types.Signatu
 			continue
 		}
 
-		allMatch, _ = ci.typeMatches(assignee, sig.Results().At(i).Type())
+		allMatch = ci.typeMatches(assignee, sig.Results().At(i).Type())
 		if !allMatch {
 			break
 		}
